#include <QtWaylandClient/private/qwayland-xdg-shell.h>

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")

namespace QtWayland {
    xdg_shell::xdg_shell(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    xdg_shell::xdg_shell(struct ::xdg_shell *obj)
        : m_xdg_shell(obj)
    {
        init_listener();
    }

    xdg_shell::xdg_shell()
        : m_xdg_shell(0)
    {
    }

    xdg_shell::~xdg_shell()
    {
    }

    void xdg_shell::init(struct ::wl_registry *registry, int id, int version)
    {
        m_xdg_shell = static_cast<struct ::xdg_shell *>(wl_registry_bind(registry, id, &xdg_shell_interface, version));
        init_listener();
    }

    void xdg_shell::init(struct ::xdg_shell *obj)
    {
        m_xdg_shell = obj;
        init_listener();
    }

    bool xdg_shell::isInitialized() const
    {
        return m_xdg_shell != 0;
    }

    const struct wl_interface *xdg_shell::interface()
    {
        return &::xdg_shell_interface;
    }

    void xdg_shell::destroy()
    {
        xdg_shell_destroy(
            m_xdg_shell);
        m_xdg_shell = 0;
    }

    void xdg_shell::use_unstable_version(int32_t version)
    {
        xdg_shell_use_unstable_version(
            m_xdg_shell,
            version);
    }

    struct ::xdg_surface *xdg_shell::get_xdg_surface(struct ::wl_surface *surface)
    {
        return xdg_shell_get_xdg_surface(
            m_xdg_shell,
            surface);
    }

    struct ::xdg_popup *xdg_shell::get_xdg_popup(struct ::wl_surface *surface, struct ::wl_surface *parent, struct ::wl_seat *seat, uint32_t serial, int32_t x, int32_t y)
    {
        return xdg_shell_get_xdg_popup(
            m_xdg_shell,
            surface,
            parent,
            seat,
            serial,
            x,
            y);
    }

    void xdg_shell::pong(uint32_t serial)
    {
        xdg_shell_pong(
            m_xdg_shell,
            serial);
    }

    void xdg_shell::xdg_shell_ping(uint32_t )
    {
    }

    void xdg_shell::handle_ping(
        void *data,
        struct ::xdg_shell *object,
        uint32_t serial)
    {
        Q_UNUSED(object);
        static_cast<xdg_shell *>(data)->xdg_shell_ping(
            serial);
    }

    const struct xdg_shell_listener xdg_shell::m_xdg_shell_listener = {
        xdg_shell::handle_ping
    };

    void xdg_shell::init_listener()
    {
        xdg_shell_add_listener(m_xdg_shell, &m_xdg_shell_listener, this);
    }

    xdg_surface::xdg_surface(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    xdg_surface::xdg_surface(struct ::xdg_surface *obj)
        : m_xdg_surface(obj)
    {
        init_listener();
    }

    xdg_surface::xdg_surface()
        : m_xdg_surface(0)
    {
    }

    xdg_surface::~xdg_surface()
    {
    }

    void xdg_surface::init(struct ::wl_registry *registry, int id, int version)
    {
        m_xdg_surface = static_cast<struct ::xdg_surface *>(wl_registry_bind(registry, id, &xdg_surface_interface, version));
        init_listener();
    }

    void xdg_surface::init(struct ::xdg_surface *obj)
    {
        m_xdg_surface = obj;
        init_listener();
    }

    bool xdg_surface::isInitialized() const
    {
        return m_xdg_surface != 0;
    }

    const struct wl_interface *xdg_surface::interface()
    {
        return &::xdg_surface_interface;
    }

    void xdg_surface::destroy()
    {
        xdg_surface_destroy(
            m_xdg_surface);
        m_xdg_surface = 0;
    }

    void xdg_surface::set_parent(struct ::xdg_surface *parent)
    {
        xdg_surface_set_parent(
            m_xdg_surface,
            parent);
    }

    void xdg_surface::set_title(const QString &title)
    {
        xdg_surface_set_title(
            m_xdg_surface,
            title.toUtf8().constData());
    }

    void xdg_surface::set_app_id(const QString &app_id)
    {
        xdg_surface_set_app_id(
            m_xdg_surface,
            app_id.toUtf8().constData());
    }

    void xdg_surface::show_window_menu(struct ::wl_seat *seat, uint32_t serial, int32_t x, int32_t y)
    {
        xdg_surface_show_window_menu(
            m_xdg_surface,
            seat,
            serial,
            x,
            y);
    }

    void xdg_surface::move(struct ::wl_seat *seat, uint32_t serial)
    {
        xdg_surface_move(
            m_xdg_surface,
            seat,
            serial);
    }

    void xdg_surface::resize(struct ::wl_seat *seat, uint32_t serial, uint32_t edges)
    {
        xdg_surface_resize(
            m_xdg_surface,
            seat,
            serial,
            edges);
    }

    void xdg_surface::ack_configure(uint32_t serial)
    {
        xdg_surface_ack_configure(
            m_xdg_surface,
            serial);
    }

    void xdg_surface::set_window_geometry(int32_t x, int32_t y, int32_t width, int32_t height)
    {
        xdg_surface_set_window_geometry(
            m_xdg_surface,
            x,
            y,
            width,
            height);
    }

    void xdg_surface::set_maximized()
    {
        xdg_surface_set_maximized(
            m_xdg_surface);
    }

    void xdg_surface::unset_maximized()
    {
        xdg_surface_unset_maximized(
            m_xdg_surface);
    }

    void xdg_surface::set_fullscreen(struct ::wl_output *output)
    {
        xdg_surface_set_fullscreen(
            m_xdg_surface,
            output);
    }

    void xdg_surface::unset_fullscreen()
    {
        xdg_surface_unset_fullscreen(
            m_xdg_surface);
    }

    void xdg_surface::set_minimized()
    {
        xdg_surface_set_minimized(
            m_xdg_surface);
    }

    void xdg_surface::xdg_surface_configure(int32_t , int32_t , wl_array *, uint32_t )
    {
    }

    void xdg_surface::handle_configure(
        void *data,
        struct ::xdg_surface *object,
        int32_t width,
        int32_t height,
        wl_array *states,
        uint32_t serial)
    {
        Q_UNUSED(object);
        static_cast<xdg_surface *>(data)->xdg_surface_configure(
            width,
            height,
            states,
            serial);
    }

    void xdg_surface::xdg_surface_close()
    {
    }

    void xdg_surface::handle_close(
        void *data,
        struct ::xdg_surface *object)
    {
        Q_UNUSED(object);
        static_cast<xdg_surface *>(data)->xdg_surface_close();
    }

    const struct xdg_surface_listener xdg_surface::m_xdg_surface_listener = {
        xdg_surface::handle_configure,
        xdg_surface::handle_close
    };

    void xdg_surface::init_listener()
    {
        xdg_surface_add_listener(m_xdg_surface, &m_xdg_surface_listener, this);
    }

    xdg_popup::xdg_popup(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    xdg_popup::xdg_popup(struct ::xdg_popup *obj)
        : m_xdg_popup(obj)
    {
        init_listener();
    }

    xdg_popup::xdg_popup()
        : m_xdg_popup(0)
    {
    }

    xdg_popup::~xdg_popup()
    {
    }

    void xdg_popup::init(struct ::wl_registry *registry, int id, int version)
    {
        m_xdg_popup = static_cast<struct ::xdg_popup *>(wl_registry_bind(registry, id, &xdg_popup_interface, version));
        init_listener();
    }

    void xdg_popup::init(struct ::xdg_popup *obj)
    {
        m_xdg_popup = obj;
        init_listener();
    }

    bool xdg_popup::isInitialized() const
    {
        return m_xdg_popup != 0;
    }

    const struct wl_interface *xdg_popup::interface()
    {
        return &::xdg_popup_interface;
    }

    void xdg_popup::destroy()
    {
        xdg_popup_destroy(
            m_xdg_popup);
        m_xdg_popup = 0;
    }

    void xdg_popup::xdg_popup_popup_done()
    {
    }

    void xdg_popup::handle_popup_done(
        void *data,
        struct ::xdg_popup *object)
    {
        Q_UNUSED(object);
        static_cast<xdg_popup *>(data)->xdg_popup_popup_done();
    }

    const struct xdg_popup_listener xdg_popup::m_xdg_popup_listener = {
        xdg_popup::handle_popup_done
    };

    void xdg_popup::init_listener()
    {
        xdg_popup_add_listener(m_xdg_popup, &m_xdg_popup_listener, this);
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
