#include <QtWaylandCompositor/private/qwayland-server-server-buffer-extension.h>

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")

namespace QtWaylandServer {
    qt_server_buffer::qt_server_buffer(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_server_buffer::qt_server_buffer(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_server_buffer::qt_server_buffer(struct ::wl_resource *resource)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(resource);
    }

    qt_server_buffer::qt_server_buffer()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_server_buffer::~qt_server_buffer()
    {
    }

    void qt_server_buffer::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    void qt_server_buffer::init(struct ::wl_resource *resource)
    {
        m_resource = bind(resource);
    }

    qt_server_buffer::Resource *qt_server_buffer::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_server_buffer::Resource *qt_server_buffer::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_server_buffer::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_server_buffer_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    const struct wl_interface *qt_server_buffer::interface()
    {
        return &::qt_server_buffer_interface;
    }

    qt_server_buffer::Resource *qt_server_buffer::server_buffer_allocate()
    {
        return new Resource;
    }

    void qt_server_buffer::server_buffer_bind_resource(Resource *)
    {
    }

    void qt_server_buffer::server_buffer_destroy_resource(Resource *)
    {
    }

    void qt_server_buffer::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_server_buffer *that = static_cast<qt_server_buffer *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_server_buffer::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_server_buffer *that = resource->server_buffer_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->server_buffer_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_server_buffer::Resource *qt_server_buffer::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Q_ASSERT_X(!wl_client_get_object(client, id), "QWaylandObject bind", QStringLiteral("binding to object %1 more than once").arg(id).toLocal8Bit().constData());
        struct ::wl_resource *handle = wl_resource_create(client, &::qt_server_buffer_interface, version, id);
        return bind(handle);
    }

    qt_server_buffer::Resource *qt_server_buffer::bind(struct ::wl_resource *handle)
    {
        Resource *resource = server_buffer_allocate();
        resource->server_buffer_object = this;

        wl_resource_set_implementation(handle, &m_qt_server_buffer_interface, resource, destroy_func);
        resource->handle = handle;
        server_buffer_bind_resource(resource);
        return resource;
    }
    qt_server_buffer::Resource *qt_server_buffer::Resource::fromResource(struct ::wl_resource *resource)
    {
        if (wl_resource_instance_of(resource, &::qt_server_buffer_interface, &m_qt_server_buffer_interface))
            return static_cast<Resource *>(resource->data);
        return 0;
    }

    const struct ::qt_server_buffer_interface qt_server_buffer::m_qt_server_buffer_interface = {
        qt_server_buffer::handle_release
    };

    void qt_server_buffer::server_buffer_release(Resource *)
    {
    }


    void qt_server_buffer::handle_release(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_server_buffer *>(r->server_buffer_object)->server_buffer_release(
            r);
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
